@@ -12,16 +12,33 @@
 #' @param registry path to the registry directory, defaults to the value of the
 #'   environment variable CORPUS_REGISTRY
 #' @examples 
-#' registry <- if (!check_pkg_registry_files()) use_tmp_registry() else get_pkg_registry()
-#' 
-#' Sys.setenv(CORPUS_REGISTRY = registry)
-#' token_no <- cl_attribute_size("REUTERS", attribute = "word", attribute_type = "p")
+#' token_no <- cl_attribute_size(
+#'   "REUTERS",
+#'   attribute = "word",
+#'   attribute_type = "p",
+#'   registry = get_tmp_registry()
+#' )
 #' corpus_positions <- seq.int(from = 0, to = token_no - 1)
-#' cl_cpos2id("REUTERS", "word", cpos = corpus_positions)
+#' cl_cpos2id(
+#'   "REUTERS",
+#'   "word",
+#'   cpos = corpus_positions,
+#'   registry = get_tmp_registry()
+#' )
 #' 
-#' places_no <- cl_attribute_size("REUTERS", attribute = "places", attribute_type = "s")
+#' places_no <- cl_attribute_size(
+#'   "REUTERS",
+#'   attribute = "places",
+#'   attribute_type = "s",
+#'   registry = get_tmp_registry()
+#' )
 #' strucs <- seq.int(from = 0, to = places_no - 1)
-#' cl_struc2str("REUTERS", "places", struc = strucs)
+#' cl_struc2str(
+#'   "REUTERS",
+#'   "places",
+#'   struc = strucs,
+#'   registry = get_tmp_registry()
+#' )
 cl_attribute_size <- function(corpus, attribute, attribute_type, registry = Sys.getenv("CORPUS_REGISTRY")){
   .cl_attribute_size(corpus = corpus, attribute = attribute, attribute_type = attribute_type, registry = registry)
 }
@@ -38,11 +55,19 @@ cl_attribute_size <- function(corpus, attribute, attribute_type, registry = Sys.
 #'   environment variable CORPUS_REGISTRY
 #' @rdname cl_lexicon_size
 #' @examples 
-#' registry <- if (!check_pkg_registry_files()) use_tmp_registry() else get_pkg_registry()
-#' Sys.setenv(CORPUS_REGISTRY = registry)
-#' lexicon_size <- cl_lexicon_size("REUTERS", p_attribute = "word")
+#' lexicon_size <- cl_lexicon_size(
+#'   "REUTERS",
+#'   p_attribute = "word",
+#'   registry = get_tmp_registry()
+#' )
+#' 
 #' token_ids <- seq.int(from = 0, to = lexicon_size - 1)
-#' cl_id2str("REUTERS", p_attribute = "word", id = token_ids)
+#' cl_id2str(
+#'   "REUTERS",
+#'   p_attribute = "word",
+#'   id = token_ids,
+#'   registry = get_tmp_registry()
+#' )
 cl_lexicon_size <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_REGISTRY")){
   .cl_lexicon_size(corpus = corpus, p_attribute = p_attribute, registry = registry)
 }
@@ -66,33 +91,60 @@ cl_lexicon_size <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_R
 #' @rdname s_attributes
 #' @name CL: s_attributes
 #' @examples
-#' registry <- if (!check_pkg_registry_files()) use_tmp_registry() else get_pkg_registry()
-#' 
 #' # get metadata for matches of token
 #' # scenario: id of the texts with occurrence of 'oil'
 #' token_to_get <- "oil"
-#' token_id <- cl_str2id("REUTERS", p_attribute = "word", str = "oil")
-#' token_cpos <- cl_id2cpos("REUTERS", p_attribute = "word", id = token_id)
-#' strucs <- cl_cpos2struc("REUTERS", s_attribute = "id", cpos = token_cpos)
+#' token_id <- cl_str2id("REUTERS", p_attribute = "word", str = "oil", get_tmp_registry())
+#' token_cpos <- cl_id2cpos("REUTERS", p_attribute = "word", id = token_id, get_tmp_registry())
+#' strucs <- cl_cpos2struc("REUTERS", s_attribute = "id", cpos = token_cpos, get_tmp_registry())
 #' strucs_unique <- unique(strucs)
-#' text_ids <- cl_struc2str("REUTERS", s_attribute = "id", struc = strucs_unique)
+#' text_ids <- cl_struc2str("REUTERS", s_attribute = "id", struc = strucs_unique, get_tmp_registry())
 #' 
 #' # get the full text of the first text with match for 'oil'
-#' left_cpos <- cl_cpos2lbound("REUTERS", s_attribute = "id", cpos = min(token_cpos))
-#' right_cpos <- cl_cpos2rbound("REUTERS", s_attribute = "id", cpos = min(token_cpos))
-#' txt <- cl_cpos2str("REUTERS", p_attribute = "word", cpos = left_cpos:right_cpos)
+#' left_cpos <- cl_cpos2lbound(
+#'   "REUTERS", s_attribute = "id",
+#'   cpos = min(token_cpos),
+#'   registry = get_tmp_registry()
+#' )
+#' right_cpos <- cl_cpos2rbound(
+#'   "REUTERS",
+#'   s_attribute = "id",
+#'   cpos = min(token_cpos),
+#'   registry = get_tmp_registry()
+#' )
+#' txt <- cl_cpos2str(
+#'   "REUTERS", p_attribute = "word",
+#'   cpos = left_cpos:right_cpos,
+#'   registry = get_tmp_registry()
+#' )
 #' fulltext <- paste(txt, collapse = " ")
 #' 
 #' # alternativ approach to achieve same result
-#' first_struc_match_oil <- cl_cpos2struc("REUTERS", s_attribute = "id", cpos = min(token_cpos))
-#' cpos_struc <- cl_struc2cpos("REUTERS", s_attribute = "id", struc = first_struc_match_oil)
-#' txt <- cl_cpos2str("REUTERS", p_attribute = "word", cpos = cpos_struc[1]:cpos_struc[2])
+#' first_struc_match_oil <- cl_cpos2struc(
+#'   "REUTERS", s_attribute = "id",
+#'   cpos = min(token_cpos),
+#'   registry = get_tmp_registry()
+#'   )
+#' cpos_struc <- cl_struc2cpos(
+#'   "REUTERS", s_attribute = "id",
+#'   struc = first_struc_match_oil,
+#'   registry = get_tmp_registry()
+#' )
+#' txt <- cl_cpos2str(
+#'   "REUTERS",
+#'   p_attribute = "word",
+#'   cpos = cpos_struc[1]:cpos_struc[2],
+#'   registry = get_tmp_registry()
+#' )
 #' fulltext <- paste(txt, collapse = " ")
 cl_cpos2struc <- function(corpus, s_attribute, cpos, registry = Sys.getenv("CORPUS_REGISTRY")){
   check_registry(registry)
   check_corpus(corpus, registry)
   check_s_attribute(corpus = corpus, registry = registry, s_attribute = s_attribute)
+  
+  if (length(cpos) == 0L) return(integer())
   check_cpos(corpus = corpus, p_attribute = "word", cpos = cpos, registry = registry)
+  
   .cl_cpos2struc(corpus = corpus, s_attribute = s_attribute, cpos = cpos, registry = registry)
 }
 
@@ -119,7 +171,10 @@ cl_cpos2lbound <- function(corpus, s_attribute, cpos, registry = Sys.getenv("COR
   check_registry(registry)
   check_corpus(corpus, registry)
   check_s_attribute(corpus = corpus, registry = registry, s_attribute = s_attribute)
+  
+  if (length(cpos) == 0L) return(integer())
   check_cpos(corpus = corpus, p_attribute = "word", cpos = cpos, registry = registry)
+  
   .cl_cpos2lbound(corpus = corpus, s_attribute = s_attribute, cpos = cpos, registry = registry)
 }
 
@@ -128,7 +183,10 @@ cl_cpos2rbound <- function(corpus, s_attribute, cpos, registry = Sys.getenv("COR
   check_registry(registry)
   check_corpus(corpus, registry)
   check_s_attribute(corpus = corpus, registry = registry, s_attribute = s_attribute)
+  
   check_cpos(corpus = corpus, p_attribute = "word", cpos = cpos, registry = registry)
+  if (length(cpos) == 0L) return(integer())
+  
   .cl_cpos2rbound(corpus = corpus, s_attribute = s_attribute, cpos = cpos, registry = registry)
 }
 
@@ -157,61 +215,60 @@ cl_cpos2rbound <- function(corpus, s_attribute, cpos, registry = Sys.getenv("COR
 #' @name CL: p_attributes
 #' @examples 
 #' # registry directory and cpos_total will be needed in examples
-#' registry <- if (!check_pkg_registry_files()) use_tmp_registry() else get_pkg_registry()
-#' Sys.setenv(CORPUS_REGISTRY = registry)
 #' cpos_total <- cl_attribute_size(
 #'   corpus = "REUTERS", attribute = "word",
-#'   attribute_type = "p", registry = registry
+#'   attribute_type = "p", registry = get_tmp_registry()
 #'   )
 #' 
 #' # decode the token stream of the corpus (the quick way)
 #' token_stream_str <- cl_cpos2str(
 #'   corpus = "REUTERS", p_attribute = "word",
 #'   cpos = seq.int(from = 0, to = cpos_total - 1),
-#'   registry = registry
+#'   registry = get_tmp_registry()
 #'   )
 #'   
 #' # decode the token stream (cpos2id first, then id2str)
 #' token_stream_ids <- cl_cpos2id(
 #'   corpus = "REUTERS", p_attribute = "word",
 #'   cpos = seq.int(from = 0, to = cpos_total - 1),
-#'   registry = registry
+#'   registry = get_tmp_registry()
 #'   )
 #' token_stream_str <- cl_id2str(
 #'   corpus = "REUTERS", p_attribute = "word",
-#'   id = token_stream_ids, registry = registry
+#'   id = token_stream_ids, registry = get_tmp_registry()
 #' )
 #' 
 #' # get corpus positions of a token
 #' token_to_get <- "oil"
 #' id_oil <- cl_str2id(
 #'   corpus = "REUTERS", p_attribute = "word",
-#'   str = token_to_get
+#'   str = token_to_get, registry = get_tmp_registry()
 #'   )
 #' cpos_oil <- cl_id2cpos <- cl_id2cpos(
 #'   corpus = "REUTERS", p_attribute = "word",
-#'   id = id_oil
+#'   id = id_oil, registry = get_tmp_registry()
 #' )
 #' 
 #' # get frequency of token
 #' oil_freq <- cl_id2freq(
-#'   corpus = "REUTERS", p_attribute = "word", id = id_oil
+#'   corpus = "REUTERS", p_attribute = "word", id = id_oil, registry = get_tmp_registry()
 #' )
 #' length(cpos_oil) # needs to be the same as oil_freq
 #' 
 #' # use regular expressions 
 #' ids <- cl_regex2id(
 #'   corpus = "REUTERS", p_attribute = "word",
-#'   regex = "M.*"
+#'   regex = "M.*", registry = get_tmp_registry()
 #' )
 #' m_words <- cl_id2str(
 #'   corpus = "REUTERS", p_attribute = "word",
-#'   id = ids
+#'   id = ids, registry = get_tmp_registry()
 #' )
 #' 
 cl_cpos2str <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_REGISTRY"), cpos){
   check_registry(registry)
   check_corpus(corpus, registry)
+  if (length(cpos) == 0L) return(integer())
   .cl_cpos2str(corpus = corpus, p_attribute = p_attribute, registry = registry, cpos = cpos)
 }
 
@@ -219,6 +276,7 @@ cl_cpos2str <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_REGIS
 cl_cpos2id <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_REGISTRY"), cpos){
   check_registry(registry)
   check_corpus(corpus, registry)
+  if (length(cpos) == 0L) return(integer())
   .cl_cpos2id(corpus = corpus, p_attribute = p_attribute, registry = registry, cpos = cpos)
 }
 
@@ -263,6 +321,15 @@ cl_id2cpos <- function(corpus, p_attribute, id, registry = Sys.getenv("CORPUS_RE
   .cl_id2cpos(corpus = corpus, p_attribute = p_attribute, id = id, registry = registry)
 }
 
+#' Load corpus.
+#' 
+#' @inheritParams cl_delete_corpus
+#' @return A `externalptr` referencing the C representation of the corpus.
+#' @export
+cl_find_corpus <- function(corpus, registry){
+  .cl_find_corpus(corpus = tolower(corpus), registry = registry)
+}
+
 #' Drop loaded corpus.
 #' 
 #' Remove a corpus from the list of loaded corpora of the corpus library (CL).
@@ -288,7 +355,7 @@ cl_id2cpos <- function(corpus, p_attribute, id, registry = Sys.getenv("CORPUS_RE
 #' cl_delete_corpus("UNGA")
 #' corpus_is_loaded("UNGA")
 cl_delete_corpus <- function(corpus, registry = Sys.getenv("CORPUS_REGISTRY")){
-  invisible(.cl_delete_corpus(corpus = corpus, registry = registry))
+  as.logical(.cl_delete_corpus(corpus = corpus, registry = registry))
 }
 
 #' Get charset of a corpus.
@@ -357,3 +424,144 @@ corpus_data_dir <- function(corpus, registry = Sys.getenv("CORPUS_REGISTRY")){
 corpus_is_loaded <- function(corpus, registry = Sys.getenv("CORPUS_REGISTRY")){
   as.logical(.corpus_is_loaded(corpus = corpus, registry = registry))
 }
+
+#' Load corpus
+#' 
+#' @inheritParams corpus_data_dir
+#' @return `TRUE` if corpus could be loaded and `FALSE` if not.
+#' @export corpus_is_loaded
+#' @examples
+#' cl_load_corpus("REUTERS")
+cl_load_corpus <- function(corpus, registry = Sys.getenv("CORPUS_REGISTRY")){
+  as.logical(.cl_load_corpus(corpus = corpus, registry = registry))
+}
+
+#' Show CL corpora
+#' 
+#' @return A `character` vector.
+#' @export corpus_is_loaded
+#' @examples
+#' cl_list_corpora()
+cl_list_corpora <- function(){
+  .cl_list_corpora()
+}
+
+############### experimental functionality ##########################
+
+#' Experimental low-level CL access.
+#' 
+#' Set of functions with same functionality as cl_* functions to improve the
+#' ease of writing code.
+#' 
+#' @name cl_rework
+#' @rdname cl_rework
+NULL
+
+#' @param corpus ID of a CWB corpus (length-one `character` vector).
+#' @param s_attribute A structural attribute (length-one `character` vector).
+#' @param registry Registry directory.
+#' @rdname cl_rework
+#' @export
+s_attr <- function(corpus, s_attribute, registry = Sys.getenv("CORPUS_REGISTRY")){
+  .s_attr(corpus = corpus, s_attribute = s_attribute, registry = registry)
+}
+
+#' @param p_attribute A positional attribute (length-one `character` vector).
+#' @rdname cl_rework
+#' @export
+p_attr <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_REGISTRY")){
+  .p_attr(corpus = corpus, p_attribute = p_attribute, registry = registry)
+}
+
+#' @param p_attr A `externalptr` referencing a p-attribute.
+#' @rdname cl_rework
+#' @export
+p_attr_size <- function(p_attr){
+  .p_attr_size(p_attr)
+}
+
+#' @param s_attr A `externalptr` referencing a p-attribute.
+#' @rdname cl_rework
+#' @export
+s_attr_size <- function(s_attr){
+  .s_attr_size(s_attr)
+}
+
+#' @rdname cl_rework
+#' @export
+p_attr_lexicon_size <- function(p_attr){
+  .p_attr_lexicon_size(p_attr)
+}
+
+#' @param cpos An `integer` vector of corpus positions.
+#' @rdname cl_rework
+#' @export
+cpos_to_struc <- function(cpos, s_attr){
+  .cpos_to_struc(s_attr = s_attr, cpos = cpos) # reverse order!
+}
+
+#' @rdname cl_rework
+#' @export
+cpos_to_str <- function(cpos, p_attr){
+  .cpos_to_str(p_attr = p_attr, cpos = cpos) # reverse order!
+}
+
+#' @rdname cl_rework
+#' @export
+cpos_to_id <- function(cpos, p_attr){
+  .cpos_to_id(p_attr = p_attr, cpos = cpos) # reverse order!
+}
+
+
+#' @param struc A length-one `integer` vector with a struc.
+#' @rdname cl_rework
+#' @export
+struc_to_cpos <- function(struc, s_attr){
+  .struc_to_cpos(s_attr = s_attr, struc = struc) # reverse order!
+}
+
+#' @rdname cl_rework
+#' @export
+struc_to_str <- function(struc, s_attr){
+  .struc_to_str(s_attr = s_attr, struc = struc) # reverse order!
+}
+
+#' @param regex A regular expression.
+#' @rdname cl_rework
+#' @export
+regex_to_id <- function(regex, p_attr){
+  .regex_to_id(p_attr = p_attr, regex = regex) # reverse order!
+}
+
+#' @param str A `character` vector.
+#' @rdname cl_rework
+#' @export
+str_to_id <- function(str, p_attr){
+  .str_to_id(p_attr = p_attr, str = str) # reverse order!
+}
+
+#' @param id An `integer` vector with token ids.
+#' @rdname cl_rework
+#' @export
+id_to_freq <- function(id, p_attr){
+  .id_to_freq(p_attr = p_attr, id = id) # reverse order!
+}
+
+#' @rdname cl_rework
+#' @export
+id_to_cpos <- function(id, p_attr){
+  .id_to_cpos(p_attr = p_attr, id = id) # reverse order!
+}
+
+#' @rdname cl_rework
+#' @export
+cpos_to_lbound <- function(cpos, s_attr){
+  .cpos_to_lbound(s_attr = s_attr, cpos = cpos) # reverse order!
+}
+
+
+#' @rdname cl_rework
+#' @export
+cpos_to_rbound <- function(cpos, s_attr){
+  .cpos_to_rbound(s_attr = s_attr, cpos = cpos) # reverse order!
+}
